package com.softslate.commerce.businessobjects.order;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.ApplicationException;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;
import com.softslate.commerce.businessobjects.email.EmailProcessor;
import com.softslate.commerce.businessobjects.events.NewOrderEvent;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.businessobjects.product.Option;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductAttribute;
import com.softslate.commerce.businessobjects.shipping.ShippingRuleProcessor;
import com.softslate.commerce.customer.core.FormUtils;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;
import com.softslate.commerce.daos.order.OrderGatewayDAO;

/**
 * Processes business logic having to do with a user's cart.
 * <code>BasicCartProcessor</code> is the default implementation of the
 * <code>CartProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.order.CartProcessor
 */
public class BasicCartProcessor extends BaseBusinessProcessor implements
		CartProcessor {

	static Log log = LogFactory.getLog(BasicCartProcessor.class);

	public static String addItemStatus = "Item added";

	public static String initializedStatus = "Initialized";

	public static String addressAddedStatus = "Address added";

	public static String pendingStatus = "Pending";

	public static String receivedStatus = "Received";

	public static String corruptedStatus = "Corrupt";

	public boolean skipDiscountsAndShipping = false;

	public Map processAddItems(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing adding items to cart.");

		// Initialize some objects
		Map results = new HashMap();
		Collection newOrderItems = new ArrayList();
		Collection productsAdded = new ArrayList();
		boolean productAdded = false;
		Collection missingAttributes = new ArrayList();
		results.put("missingAttributes", missingAttributes);

		// If an Order for this user has not been recorded in the database,
		// create a record now and add it to the User.
		if (getUser().getOrder() == null
				|| getUser().getOrder().getOrderID() == 0) {
			if (log.isInfoEnabled())
				log.info("Initializing user's Order.");
			initializeCart(parameters);
		}
		try {
			// Query the database for the products being added
			String[] orderProductCodes = BeanUtils.getArrayProperty(parameters,
					"orderProductCodes");
			if (log.isDebugEnabled())
				log.debug("Querying for product info.");
			OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
					.createDAO(OrderGatewayDAO.class);
			Collection productInfo = orderGatewayDAO
					.getProductInfoFromCodes(orderProductCodes);
			if (log.isDebugEnabled())
				log.debug("Looping through items to add.");

			// Loop through the products being added
			for (int i = 0; i < orderProductCodes.length; i++) {

				// Find the quantity and skip the insert if it's not > 0.
				int quantity = getOrderItemQuantity(parameters, i);
				if (quantity <= 0) {
					continue;
				}

				// Find the Product object that was queried for above
				Product product = null;
				Iterator pi = productInfo.iterator();
				while (pi.hasNext()) {
					Product thisProduct = (Product) pi.next();
					if (thisProduct.getCode().equals(orderProductCodes[i])) {
						product = thisProduct;
						break;
					}
				}
				if (product == null) {
					continue;
				}

				// Create and populate a new OrderItem.
				if (log.isDebugEnabled())
					log.debug("Creating orderItem for code: "
							+ product.getCode());
				OrderItem orderItem =  getBusinessObjectFactory()
						.createObject(OrderItem.class);
				BeanUtils.copyProperties(orderItem, product);
				orderItem.setOrder(getUser().getOrder());
				orderItem
						.setOrderDelivery(getUser().getOrder().firstDelivery());
				orderItem.setImage((String) product.getSmallImage());
				orderItem.setProductWeight(product.getWeight());
				orderItem.setQuantity(quantity);
				newOrderItems.add(orderItem);
				productsAdded.add(product);
				productAdded = true;

				// Process the product's attributes
				processItemAttributes(parameters, product, missingAttributes,
						newOrderItems, orderItem, i);
			}

			// If no product was found with a quantity, throw an error.
			if (!productAdded) {
				throw new ApplicationException("Product not found: "
						+ orderProductCodes[0]);
			}

			// Process the newly created OrderItems
			results.putAll(processAddItems(newOrderItems));
			results.put("newOrderItems", newOrderItems);
			results.put("productsAdded", productsAdded);
		} catch (ApplicationException ae) {
			throw ae;
		} catch (Exception e) {
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(corruptedStatus);
			}
			throw e;
		}
		return results;
	}

	public Map processAddItems(Collection newOrderItems) throws Exception {
		Map results = new HashMap();

		// Process the quantities of the order items - sets the totals
		processQuantities(newOrderItems);
		Map newResults = new HashMap();

		// Process inventory - might affect order item quantities
		String trackInventory = (String) getSettings().getValue(
				"trackInventory");
		if (trackInventory.equals("1")) {
			InventoryProcessor inventoryProcessor =  getBusinessObjectFactory()
					.createObject(InventoryProcessor.class);
			newResults = inventoryProcessor.processAddItems(newOrderItems);
			if (newResults != null) {
				results.putAll(newResults);
			}
		}

		// The inventory check might have removed all items
		if (!newOrderItems.isEmpty()) {
			// Compute the order and delivery totals, and add formatting
			processTotalsAndFormatting(newOrderItems);
			// Add to the first delivery
			getUser().getOrder().firstDelivery().getOrderItemCollection()
					.addAll(newOrderItems);
			// Set the statuses, and if the user is logged in, copy in the
			// customerID and userName
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(addItemStatus);
				getUser().getOrder().firstDelivery().setStatus(addItemStatus);
			}
			if (getUser().getCustomer() != null) {
				getUser().getOrder().setCustomer(getUser().getCustomer());
				getUser().getOrder().setUserName(
						getUser().getCustomer().getUserName());
			}

			// Discounts and shipping are skipped if we are in the middle of
			// editing an item
			if (!skipDiscountsAndShipping) {
				// Process discounts - might affect order totals
				CartDiscountProcessor discountProcessor =  getBusinessObjectFactory()
						.createObject(CartDiscountProcessor.class);
				newResults = discountProcessor.processAddItems(newOrderItems);
				if (newResults != null) {
					results.putAll(newResults);
				}
				// Process shipping rules/packages
				ShippingRuleProcessor shippingRuleProcessor =  getBusinessObjectFactory()
						.createObject(ShippingRuleProcessor.class);
				newResults = shippingRuleProcessor
						.processAddItems(newOrderItems);
				if (newResults != null) {
					results.putAll(newResults);
				}
			}
		}

		// Method hook for other processing
		newResults = postProcessAddItems(newOrderItems);
		if (newResults != null) {
			results.putAll(newResults);
		}
		// Process the new items in the database
		// This will also deactivate products and decrement SKUs
		if (!newOrderItems.isEmpty()) {
			OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
					.createDAO(OrderGatewayDAO.class);
			orderGatewayDAO
					.processOrderItems(getUser(), newOrderItems, results);
		}

		// Method hook for any change made to the cart
		newResults = postProcessCartChanged();
		if (newResults != null) {
			results.putAll(newResults);
		}
		return results;
	}

	public void initializeCart() throws Exception {
		initializeCart(null);
	}

	public void initializeCart(Map parameters) throws Exception {
		// Initialize order and order delivery objects.
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		OrderDelivery orderDelivery =  getBusinessObjectFactory()
				.createObject(OrderDelivery.class);
		if (parameters != null) {
			BeanUtils.copyProperties(order, parameters);
			BeanUtils.copyProperties(orderDelivery, parameters);
		}
		order.setOrderDeliveryCollection(new LinkedHashSet());
		order.getOrderDeliveryCollection().add(orderDelivery);
		orderDelivery.setOrder(order);
		orderDelivery.setOrderItemCollection(new LinkedHashSet());
		if (getUser().getOrder() == null
				|| getUser().getOrder().getCompleted() == null) {
			order.setStatus(initializedStatus);
			orderDelivery.setStatus(initializedStatus);
		}
		getUser().setOrder(order);
		try {
			OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
					.createDAO(OrderGatewayDAO.class);
			orderGatewayDAO.insertOrderAndDelivery(order, orderDelivery);
		} catch (Exception e) {
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(corruptedStatus);
			}
			log.error("User's session is corrupted: " + e.toString(), e);
			throw e;
		}
		return;
	}

	public void processQuantities(Collection newOrderItems) throws Exception {
		if (newOrderItems.isEmpty()) {
			return;
		}
		Iterator i = newOrderItems.iterator();
		while (i.hasNext()) {
			OrderItem oi = (OrderItem) i.next();
			if (oi.getUnitPrice() == null) {
				oi.setUnitPrice(new Double(0));
			}
			oi.setTotal(new Double(oi.getUnitPrice().doubleValue()
					* oi.getQuantity()));
			if (oi.getProductWeight() == null) {
				oi.setProductWeight(new Double(0));
			}
			oi.setWeight(new Double(oi.getProductWeight().doubleValue()
					* oi.getQuantity()));
			if (oi.getCostTotal() == null) {
				oi.setCostTotal(new Double(0));
			}
			oi.setCostTotal(new Double(oi.getUnitCost().doubleValue()
					* oi.getQuantity()));
			Collection oias = oi.getOrderItemAttributes();
			if (oias != null) {
				Iterator j = oias.iterator();
				while (j.hasNext()) {
					OrderItemAttribute oia = (OrderItemAttribute) j.next();
					// Add the total and weight to the order item
					double totalDouble = oia.getTotal().doubleValue()
							* oi.getQuantity();
					double weightDouble = oia.getWeight().doubleValue()
							* oi.getQuantity();
					oi.setTotal(new Double(oi.getTotal().doubleValue()
							+ totalDouble));
					oi.setWeight(new Double(oi.getWeight().doubleValue()
							+ weightDouble));
					double totalCostDouble = oia.getCostTotal().doubleValue()
							* oi.getQuantity();
					oi.setCostTotal(new Double(oi.getCostTotal().doubleValue()
							+ totalCostDouble));
				}
			}
		}
	}

	public void processTotalsAndFormatting(Collection orderItems)
			throws Exception {
		if (orderItems.isEmpty()) {
			return;
		}
		Iterator i = orderItems.iterator();
		while (i.hasNext()) {
			OrderItem oi = (OrderItem) i.next();
			// Update totals
			updateOrderAndDeliveryTotals(getUser().getOrder().firstDelivery(),
					oi);
			// Add formatting
			oi.addFormatting(getUser().getLocale(), (String) getSettings()
					.getValue("currencyCode"));
		}
		// Add formatting properties to the OrderDelivery
		getUser()
				.getOrder()
				.firstDelivery()
				.addFormatting(getUser().getLocale(),
						(String) getSettings().getValue("currencyCode"));
		// Add formatting properties to the Order
		getUser().getOrder().addFormatting(getUser().getLocale(),
				(String) getSettings().getValue("currencyCode"));
	}

	/**
	 * Given an <code>OrderItem</code>, updates the totals of the user's
	 * <code>Order</code> and the <code>OrderDelivery</code> the
	 * <code>OrderItem</code> is under. Calculates new totals for the quantity,
	 * price total, and weight total.
	 * 
	 * @param orderDelivery
	 *            The order delivery that contains the new
	 *            <code>OrderItem</code>.
	 * @param orderItem
	 *            The newly-added order item whose price must be added to the
	 *            order's total and the delivery's total.
	 */
	public void updateOrderAndDeliveryTotals(OrderDelivery orderDelivery,
			OrderItem orderItem) {
		if (log.isDebugEnabled())
			log.debug("Updating order totals in memory.");
		// Add the item's current totals to the order
		if (getUser().getOrder().getSubtotal() == null) {
			getUser().getOrder().setSubtotal(new Double(0));
		}
		if (getUser().getOrder().getWeight() == null) {
			getUser().getOrder().setWeight(new Double(0));
		}
		if (getUser().getOrder().getQuantity() == null) {
			getUser().getOrder().setQuantity(new Integer(0));
		}
		if (getUser().getOrder().getTaxableSubtotal() == null) {
			getUser().getOrder().setTaxableSubtotal(new Double(0));
		}
		getUser().getOrder().setSubtotal(
				new Double(getUser().getOrder().getSubtotal().doubleValue()
						+ orderItem.getTotal().doubleValue()));
		getUser().getOrder().setWeight(
				new Double(getUser().getOrder().getWeight().doubleValue()
						+ orderItem.getWeight().doubleValue()));
		getUser().getOrder().setQuantity(
				new Integer(getUser().getOrder().getQuantity().intValue()
						+ orderItem.getQuantity()));
		// Add the item's current totals to the order delivery
		if (orderDelivery.getSubtotal() == null) {
			orderDelivery.setSubtotal(new Double(0));
		}
		if (orderDelivery.getWeight() == null) {
			orderDelivery.setWeight(new Double(0));
		}
		if (orderDelivery.getQuantity() == null) {
			orderDelivery.setQuantity(new Integer(0));
		}
		if (orderDelivery.getTaxableSubtotal() == null) {
			orderDelivery.setTaxableSubtotal(new Double(0));
		}
		orderDelivery.setSubtotal(new Double(orderDelivery.getSubtotal()
				.doubleValue() + orderItem.getTotal().doubleValue()));
		orderDelivery.setWeight(new Double(orderDelivery.getWeight()
				.doubleValue() + orderItem.getWeight().doubleValue()));
		orderDelivery.setQuantity(new Integer(orderDelivery.getQuantity()
				.intValue() + orderItem.getQuantity()));
		return;
	}

	/**
	 * Retrieves the value submitted for an attribute of a given product, as
	 * part of a request to add or edit an item in the user's cart. Uses the
	 * <code>orderAttributes</code> property (which is a <code>HashMap</code>)
	 * of the incoming <code>Map</code> to look up the attribute's value. The
	 * key for the attribute is formed like so:
	 * <code>productCode|attributeCode</code>.
	 * 
	 * @param parameters
	 *            A <code>Map</code> representing the items and properties
	 *            submitted to add or edit items in the user's cart.
	 * @param productCode
	 *            The code of the product corresponding to the attribute.
	 * @param attributeCode
	 *            The code of the attribute whose value is being looked up.
	 * @param index
	 *            The index of the submitted array of product codes
	 *            corresponding to the item being added.
	 * @return The submitted value of the attribute corresponding to the
	 *         <code>attibuteCode</code>, or <code>null</code>, if the attribute
	 *         is not found.
	 * @throws Exception
	 */
	public String getOrderAttribute(Map parameters, String productCode,
			String attributeCode, int index) throws Exception {
		Map orderAttributes = (Map) parameters.get("orderAttributes");
		String key = productCode + "|" + attributeCode;
		String value = null;
		if (orderAttributes != null && orderAttributes.containsKey(key)) {
			String[] values = (String[]) orderAttributes.get(key);
			if (values != null && values.length > index) {
				value = values[index];
			} else if (values != null && values.length > 0) {
				value = values[0];
			}
		}
		return value;
	}

	/**
	 * Finds the attributes identified in the incoming parameters and creates
	 * OrderItemAttributes to be added to the OrderItem.
	 * 
	 * @param parameters
	 * @param product
	 * @param missingAttributes
	 * @param newOrderItems
	 * @param orderItem
	 * @param index
	 * @throws Exception
	 */
	public void processItemAttributes(Map parameters, Product product,
			Collection missingAttributes, Collection newOrderItems,
			OrderItem orderItem, int index) throws Exception {
		if (product.getProductAttributes() != null) {
			Iterator j = product.getProductAttributes().iterator();
			while (j.hasNext()) {
				ProductAttribute pa = (ProductAttribute) j.next();
				Attribute a = pa.getAttribute();
				if (!a.getAttributeIsActive()) {
					continue;
				}
				String attributeValue = getOrderAttribute(parameters,
						product.getCode(), a.getAttributeCode(), index);
				if (a.getAttributeIsRequired()
						&& (attributeValue == null || attributeValue.length() == 0)) {
					// Attribute is required but was not submitted
					if (log.isDebugEnabled())
						log.debug("Required attribute: " + a.getAttributeName()
								+ " Product code: " + product.getCode());
					Map missing = BeanUtils.describe(product);
					missing.putAll(BeanUtils.describe(a));
					missingAttributes.add(missing);
					newOrderItems.remove(orderItem);
				} else if (a.getAttributeOptions() == null
						|| a.getAttributeOptions().isEmpty()
						|| (a.getAttributeType() != null && (a
								.getAttributeType().equals("yesNo")
								|| a.getAttributeType().equals("compactInput") || a
								.getAttributeType().equals("expandedInput")))) {
					// Attribute does not have any options (ie a text
					// box). Only add it if it was submitted.
					if (attributeValue != null && attributeValue.length() > 0) {
						if (log.isDebugEnabled())
							log.debug("Adding free form attribute: "
									+ a.getAttributeCode() + "="
									+ attributeValue);
						processItemAttribute(orderItem, a, null, attributeValue);
					}
				} else if (a.getAttributeOptions() != null) {
					boolean invalidValueSubmitted = true;
					Iterator k = a.getAttributeOptions().iterator();
					while (k.hasNext()) {
						Option o = (Option) k.next();
						if (!o.getOptionIsActive()) {
							continue;
						}
						if ((attributeValue == null || attributeValue.length() == 0)
								&& a.getDefaultOptionID() != null
								&& a.getDefaultOptionID().intValue() == o.getOptionID()) {
							// Attribute was not submitted but we are on
							// the default option
							attributeValue = o.getOptionCode();
							if (log.isDebugEnabled())
								log.debug("Adding default attribute: "
										+ a.getAttributeCode() + "="
										+ attributeValue);
							processItemAttribute(orderItem, a, o,
									attributeValue);
							invalidValueSubmitted = false;
						} else if (attributeValue != null
								&& attributeValue.equals(o.getOptionCode())) {
							// Attribute was submitted and this is the
							// matching option
							if (log.isDebugEnabled())
								log.debug("Adding selected attribute: "
										+ a.getAttributeCode() + "="
										+ attributeValue);
							processItemAttribute(orderItem, a, o,
									attributeValue);
							invalidValueSubmitted = false;
						}
					}
					// Attribute is required but an invalid value
					// was submitted - not found among the options
					if (invalidValueSubmitted && a.getAttributeIsRequired()) {
						if (log.isDebugEnabled())
							log.debug("Invalid value submitted for attribute: "
									+ a.getAttributeName() + " Value: "
									+ attributeValue);
						Map missing = BeanUtils.describe(product);
						missing.putAll(BeanUtils.describe(a));
						missingAttributes.add(missing);
						newOrderItems.remove(orderItem);
					}
				}
			}
		}
	}

	/**
	 * Adds an attribute to a given <code>OrderItem</code>. Adds the attribute's
	 * total and weight to the order item's.
	 * 
	 * @param orderItem
	 *            The <code>OrderItem</code> that is having an attribute added
	 *            to it.
	 * @param attribute
	 *            The <code>Attribute</code> being added to the order item
	 * @param option
	 *            The <code>Option</code> of the above attribute that is being
	 *            added to the order item. If no option is relevant, this
	 *            parameter may be null.
	 * @param attributeValue
	 *            The value of the attribute being added to the
	 *            <code>OrderItem</code>. Corresponds to the option's code, or a
	 *            free form value if no option is relevant.
	 */

	public void processItemAttribute(OrderItem orderItem, Attribute attribute,
			Option option, String attributeValue) throws Exception {
		// Copy the attribute and option's properties into the new oia.
		OrderItemAttribute oia =  getBusinessObjectFactory()
				.createObject(OrderItemAttribute.class);
		BeanUtils.copyProperties(oia, attribute);
		oia.setAttributeImage(attribute.getAttributeSmallImage());
		if (option != null) {
			BeanUtils.copyProperties(oia, option);
			oia.setOptionImage(option.getOptionSmallImage());
		}
		oia.setAttributeValue(attributeValue);
		// Compute the totals for the new oia
		double oiaTotal = 0;
		if (attribute != null && attribute.getAttributeUnitPrice() != null) {
			oiaTotal = oiaTotal
					+ attribute.getAttributeUnitPrice().doubleValue();
		}
		if (option != null && option.getOptionUnitPrice() != null) {
			oiaTotal = oiaTotal + option.getOptionUnitPrice().doubleValue();
		}
		double oiaWeight = 0;
		if (attribute != null && attribute.getAttributeWeight() != null) {
			oiaWeight = oiaWeight
					+ attribute.getAttributeWeight().doubleValue();
		}
		if (option != null && option.getOptionWeight() != null) {
			oiaWeight = oiaWeight + option.getOptionWeight().doubleValue();
		}
		double oiaCostTotal = 0;
		if (attribute != null && attribute.getAttributeUnitCost() != null) {
			oiaCostTotal = oiaCostTotal
					+ attribute.getAttributeUnitCost().doubleValue();
		}
		if (option != null && option.getOptionUnitCost() != null) {
			oiaCostTotal = oiaCostTotal
					+ option.getOptionUnitCost().doubleValue();
		}
		oia.setTotal(new Double(oiaTotal));
		oia.setWeight(new Double(oiaWeight));
		oia.setCostTotal(new Double(oiaCostTotal));
		// Add the new oia's total and weight to the order item
		double oiTotal = oiaTotal * orderItem.getQuantity();
		double oiWeight = oiaWeight * orderItem.getQuantity();
		double oiCostTotal = oiaCostTotal * orderItem.getQuantity();
		if (orderItem.getTotal() == null) {
			orderItem.setTotal(new Double(0));
		}
		if (orderItem.getWeight() == null) {
			orderItem.setWeight(new Double(0));
		}
		if (orderItem.getCostTotal() == null) {
			orderItem.setCostTotal(new Double(0));
		}
		orderItem.setTotal(new Double(orderItem.getTotal().doubleValue()
				+ oiTotal));
		orderItem.setWeight(new Double(orderItem.getWeight().doubleValue()
				+ oiWeight));
		orderItem.setCostTotal(new Double(orderItem.getCostTotal()
				.doubleValue() + oiCostTotal));
		// Add the oia to the order item's collection.
		if (orderItem.getOrderItemAttributes() == null) {
			orderItem.setOrderItemAttributes(new LinkedHashSet());
		}
		orderItem.getOrderItemAttributes().add(oia);
		oia.setOrderItem(orderItem);
		return;
	}

	/**
	 * Looks up the submitted quantity of a given product contained in a request
	 * to add or edit a user's cart item. Uses the
	 * <code>orderProductQuantities</code> property (which is a
	 * <code>String[]</code>) of the incoming <code>Map</code> to look up the
	 * value.
	 * 
	 * @param parameters
	 *            A <code>Map</code> representing the items and properties being
	 *            added or edited.
	 * @param index
	 *            The index of the submitted array of product quantities
	 *            corresponding to the item being added.
	 * @return An int representing the submitted quantity of the user's cart
	 *         item
	 * @throws Exception
	 */
	public int getOrderItemQuantity(Map parameters, int index) throws Exception {
		String[] orderProductQuantities = BeanUtils.getArrayProperty(
				parameters, "orderProductQuantities");
		int quantity = 0;
		try {
			quantity = Integer.parseInt(orderProductQuantities[index]);
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug("Bad quantity value found: "
						+ orderProductQuantities[index] + " ...ignoring.");
		}
		return quantity;
	}

	public Map processEditItem(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing editing an item or items in the cart.");
		Map map = new HashMap();

		// Add the item(s) first
		String[] orderProductQuantities = (String[]) parameters
				.get("orderProductQuantities");
		if (orderProductQuantities != null
				&& this.utils().stringArrayValuesGreaterThan(
						orderProductQuantities, 0)) {
			String skipDiscountsAndShippingOnItemEdit = (String) getSettings()
					.getValue("skipDiscountsAndShippingOnItemEdit");
			if (skipDiscountsAndShippingOnItemEdit != null
					&& skipDiscountsAndShippingOnItemEdit.equals("1")) {
				skipDiscountsAndShipping = true;
			}
			map = processAddItems(parameters);
		}

		// Now remove the item(s)
		// The parameters either contain a Collection named "orderItems" or a
		// single "orderItem"
		boolean rejectedItems = map.get("rejectedOrderItems") != null
				&& ((ArrayList) map.get("rejectedOrderItems")).size() > 0;
		Map newResults = null;
		Collection orderItems = (Collection) parameters.get("orderItems");
		if (orderItems == null || orderItems.isEmpty()) {
			orderItems = new ArrayList();
			OrderItem orderItem = (OrderItem) parameters.get("orderItem");
			if (orderItem != null) {
				orderItems.add(orderItem);
			}
		}
		if (orderItems == null || orderItems.isEmpty()) {
			throw new ApplicationException(
					"Can't find the order item to remove.");
		}
		Iterator ois = orderItems.iterator();
		while (ois.hasNext()) {
			OrderItem orderItem = (OrderItem) ois.next();

			// Remove the item if there are no reject items OR this item's code
			// is among the newly added items above.
			boolean removeItem = !rejectedItems;
			if (!removeItem) {
				Collection newOrderItems = (Collection) map
						.get("newOrderItems");
				Iterator i = newOrderItems.iterator();
				while (i.hasNext()) {
					OrderItem noi = (OrderItem) i.next();
					if (noi.getCode().equals(orderItem.getCode())) {
						removeItem = true;
						break;
					}
				}
			}
			if (removeItem) {

				// Detach discounts from the item.
				Collection discounts = orderItem.getOrderDiscounts();
				if (discounts != null) {
					Iterator i = discounts.iterator();
					while (i.hasNext()) {
						OrderDiscount od = (OrderDiscount) i.next();
						od.setOrderItem(null);
					}
				}
				// Detach shipping rules from the item.
				Collection rules = orderItem.getOrderShippingRules();
				if (rules != null) {
					Iterator i = rules.iterator();
					while (i.hasNext()) {
						OrderShippingRule osr = (OrderShippingRule) i.next();
						osr.setOrderItem(null);
					}
				}

				// Now remove the item
				newResults = processRemoveItem(orderItem);
				if (newResults != null) {
					map.putAll(newResults);
				}
			}
		}

		// Process discounts again to account to add discounts back in
		CartDiscountProcessor discountProcessor =  getBusinessObjectFactory()
				.createObject(CartDiscountProcessor.class);
		newResults = discountProcessor.processAddItems(orderItems);
		if (newResults != null) {
			map.putAll(newResults);
		}

		// Redo the shipping rules. We send the new items collection here as
		// well, but never use them as we just grab the order item
		// collection directly from the user
		ShippingRuleProcessor shippingRuleProcessor =  getBusinessObjectFactory()
				.createObject(ShippingRuleProcessor.class);
		newResults = shippingRuleProcessor.processEditItem(orderItems);
		if (newResults != null) {
			map.putAll(newResults);
		}

		// Method hook for any change made to the cart
		newResults = postProcessCartChanged();
		if (newResults != null) {
			map.putAll(newResults);
		}

		return map;
	}

	public Map processRemoveItem(OrderItem orderItem) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing removing an item from the cart.");
		try {
			// Decrement totals of the Order and OrderDelivery.
			if (log.isDebugEnabled())
				log.debug("Subtracting cart totals.");
			getUser().getOrder().setSubtotal(
					new Double(getUser().getOrder().getSubtotal().doubleValue()
							- orderItem.getTotal().doubleValue()));
			getUser().getOrder().setWeight(
					new Double(getUser().getOrder().getWeight().doubleValue()
							- orderItem.getWeight().doubleValue()));
			getUser().getOrder().setQuantity(
					new Integer(getUser().getOrder().getQuantity().intValue()
							- orderItem.getQuantity()));
			getUser()
					.getOrder()
					.firstDelivery()
					.setSubtotal(
							new Double(getUser().getOrder().firstDelivery()
									.getSubtotal().doubleValue()
									- orderItem.getTotal().doubleValue()));
			getUser()
					.getOrder()
					.firstDelivery()
					.setWeight(
							new Double(getUser().getOrder().firstDelivery()
									.getWeight().doubleValue()
									- orderItem.getWeight().doubleValue()));
			getUser()
					.getOrder()
					.firstDelivery()
					.setQuantity(
							new Integer(getUser().getOrder().firstDelivery()
									.getQuantity().intValue()
									- orderItem.getQuantity()));
			getUser().getOrder().firstDelivery().getOrderItemCollection()
					.remove(orderItem);
			// Add formatting to the memory objects.
			orderItem.addFormatting(getUser().getLocale(),
					(String) getSettings().getValue("currencyCode"));
			getUser()
					.getOrder()
					.firstDelivery()
					.addFormatting(getUser().getLocale(),
							(String) getSettings().getValue("currencyCode"));
			getUser().getOrder().addFormatting(getUser().getLocale(),
					(String) getSettings().getValue("currencyCode"));
			// Now remove from the database.
			OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
					.createDAO(OrderGatewayDAO.class);
			orderGatewayDAO.removeOrderItem(getUser().getOrder(), getUser()
					.getOrder().firstDelivery(), orderItem);
		} catch (Exception e) {
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(corruptedStatus);
			}
			throw e;
		}
		return null;
	}

	public Map processCheckoutAddresses(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating order with addresses.");
		try {
			if (getUser().getCustomMap() == null) {
				getUser().setCustomMap(new HashMap());
			}

			// Reset address book payment vars, if they are re-doing the
			// addresses page, these need to be reset
			getUser().getCustomMap().put("usingBillingAddressBookID", 0);
			getUser().getCustomMap().put("usingBillingAddressBookUID", "");
			getUser().getCustomMap().put("newBillingAddressExisted", "0");
			getUser().getCustomMap().put("newDeliveryAddressExisted", "0");
			if (getUser().getOrder() != null) {
				getUser().getOrder().setPaymentProfileID(null);
			}
			String allowAddressBook = (String) getSettings().getValue("allowAddressBook");
			String maxCustomerAddressBook = (String) getSettings().getValue("maxCustomerAddressBook");
			int maxCustomerAddrBk = 1;
			try {
				maxCustomerAddrBk = Integer.parseInt(maxCustomerAddressBook);
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Invalid maximum customer address book number, using a default of 1!");
				}
			}
			CustomerAddressProcessor customerAddressProcessor =  getBusinessObjectFactory()
					.createObject(CustomerAddressProcessor.class);
			boolean usingAddressBook = false;
			String customerAddressIDstr = (String) parameters.get("customerAddressID");
			int customerAddressID = 0;
			String customerBillingAddressIDstr = (String) parameters.get("customerBillingAddressID");
			int customerBillingAddressID = 0;
			if ("1".equals(allowAddressBook) && getUser() != null && getUser().getCustomer() != null) {
				usingAddressBook = true;

				// Keep track of which address book id they choose
				try {
					customerAddressID = Integer.parseInt(customerAddressIDstr);
				} catch (Exception ex) {
					if (log.isInfoEnabled())
						log.info("Invalid customerAddressID: " + customerAddressIDstr);
				}
				try {
					customerBillingAddressID = Integer.parseInt(customerBillingAddressIDstr);
				} catch (Exception ex) {
					if (log.isInfoEnabled())
						log.info("Invalid customerBillingAddressID: " + customerBillingAddressIDstr);
				}
			}

			// Update the order and first delivery, in memory
			if (parameters != null) {
				BeanUtils.copyProperties(getUser().getOrder(), parameters);
				if (usingAddressBook) {
					boolean addBillingAddress = false;
					boolean addShippingAddress = false;
					if (customerAddressID > 0) {
						CustomerAddress customerAddress = customerAddressProcessor.findCustomerAddress(getUser()
								.getCustomer().getCustomerAddressCollection(), customerAddressID);
						if (customerAddress != null) {
							BeanUtils.copyProperties(getUser().getOrder().firstDelivery(), customerAddress);
						} else {
							// cant find chosen address, bad die here for now
							throw new Exception("Address Book Error");
						}
					} else {
						// Adding a new address using the form, use it
						BeanUtils.copyProperties(getUser().getOrder().firstDelivery(), parameters);
						if (parameters.get("addToAddressBook") != null
								&& parameters.get("addToAddressBook").toString().equals("1")
								&& StringUtils.isBlank((String) parameters.get("customerAddressID"))) {
							// Add to address book
							int addressCount = 0;
							Collection addresses = getUser().getCustomer().getCustomerAddressCollection();
							if (addresses != null && addresses.size() > 0) {
								addressCount = addresses.size();
							}
							// Just skip adding it if they are over, no worries
							if (addressCount < maxCustomerAddrBk) {
								addShippingAddress = true;
							}
						}
					}

					// Billing address
					if (customerBillingAddressID > 0) {
						CustomerAddress customerAddress = customerAddressProcessor.findCustomerAddress(getUser()
								.getCustomer().getCustomerAddressCollection(), customerBillingAddressID);
						if (customerAddress != null) {
							getUser().getCustomMap().put("usingBillingAddressBookID",
									customerAddress.getCustomerAddressID());
							BeanUtils.copyProperties(getUser().getOrder(), customerAddress);
							if (StringUtils.isNotBlank(customerAddress.getPaymentToken())) {
								getUser().getOrder().setPaymentProfileID(customerAddress.getPaymentToken());
							}
						} else {
							// cant find chosen address, bad die here for now
							throw new Exception("Address Book Error");
						}
					} else {
						// Adding a new address using the form, use it
						BeanUtils.copyProperties(getUser().getOrder(), parameters);
						if (parameters.get("addToBillingAddressBook") != null
								&& parameters.get("addToBillingAddressBook").toString().equals("1")
								&& StringUtils.isBlank(customerBillingAddressIDstr)) {
							// Add to address book
							int addressCount = 0;
							Collection addresses = getUser().getCustomer().getCustomerAddressCollection();
							if (addresses != null && addresses.size() > 0) {
								addressCount = addresses.size();
							}
							// Just skip adding it if they are over, no worries
							if (addressCount < maxCustomerAddrBk) {
								addBillingAddress = true;
							}
						}
					}

					if (addShippingAddress && addBillingAddress) {
						Map deliveryMap = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(parameters,
								"delivery", false);
						Map billingMap = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(parameters,
								"billing", false);

						// Add address book
						Map deliveryAddressInfo = customerAddressProcessor.addAddressBookAddress(deliveryMap, false);
						if (deliveryAddressInfo != null && deliveryAddressInfo.containsKey("addressExisted")) {
							// Did not add an address, because an existing one
							// matched
							BeanUtils.copyProperties(getUser().getOrder().firstDelivery(), deliveryAddressInfo);
							getUser().getCustomMap().put("newDeliveryAddressExisted", "1");
						}

						billingMap.put("isBilling", true);
						Map billingAddressInfo = customerAddressProcessor.addAddressBookAddress(billingMap, true);

						if (billingAddressInfo != null) {
							if (billingAddressInfo.containsKey("addressExisted")) {
								// Did not add an address, because an existing
								// one matched
								getUser().getCustomMap().put("newBillingAddressExisted", "1");
								getUser().getCustomMap().put("usingBillingAddressBookID",
										billingAddressInfo.get("customerAddressID"));
								BeanUtils.copyProperties(getUser().getOrder(), billingAddressInfo);
								if (StringUtils.isNotBlank((String) billingAddressInfo.get("paymentToken"))) {
									getUser().getOrder().setPaymentProfileID(
											(String) billingAddressInfo.get("paymentToken"));
								}
							} else {
								getUser().getCustomMap().put("usingBillingAddressBookUID",
										(String) billingAddressInfo.get("identifier"));
							}
						}

					} else if (addShippingAddress) {
						Map deliveryMap = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(parameters,
								"delivery", false);
						Map deliveryAddressInfo = customerAddressProcessor.addAddressBookAddress(deliveryMap, true);
						if (deliveryAddressInfo != null && deliveryAddressInfo.containsKey("addressExisted")) {
							// Did not add an address, because an existing one
							// matched
							BeanUtils.copyProperties(getUser().getOrder().firstDelivery(), deliveryAddressInfo);
							getUser().getCustomMap().put("newDeliveryAddressExisted", "1");
						}
					} else if (addBillingAddress) {
						Map billingMap = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(parameters,
								"billing", false);
						billingMap.put("isBilling", true);
						Map billingAddressInfo = customerAddressProcessor.addAddressBookAddress(billingMap, true);
						if (billingAddressInfo != null) {
							if (billingAddressInfo.containsKey("addressExisted")) {
								// Did not add an address, because an existing
								// one matched
								getUser().getCustomMap().put("newBillingAddressExisted", "1");
								getUser().getCustomMap().put("usingBillingAddressBookID",
										billingAddressInfo.get("customerAddressID"));
								BeanUtils.copyProperties(getUser().getOrder(), billingAddressInfo);
								if (StringUtils.isNotBlank((String) billingAddressInfo.get("paymentToken"))) {
									getUser().getOrder().setPaymentProfileID(
											(String) billingAddressInfo.get("paymentToken"));
								}
							} else {
								getUser().getCustomMap().put("usingBillingAddressBookUID",
										(String) billingAddressInfo.get("identifier"));
							}
						}
					}

				} else {
					BeanUtils.copyProperties(getUser().getOrder()
							.firstDelivery(), parameters);
				}
				if (parameters.get("useRecipientsEmail") != null && parameters.get("useRecipientsEmail").equals(Boolean.TRUE)) {
					getUser().getOrder().setBillingEmail1((String) parameters.get("deliveryEmail1"));
				}
			}
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(addressAddedStatus);
				getUser().getOrder().firstDelivery().setStatus(pendingStatus);
			}
			// Update the customer and customer address, if called for, in
			// memory
			CustomerGatewayDAO customerGatewayDAO = (CustomerGatewayDAO) getDaoFactory().createDAO(
					"customerGatewayDAOImplementer");
			CustomerAddress firstAddress = null;
			String updateCustomer = (String) parameters.get("updateCustomer");
			boolean isDeliveryAddressNew = false;
			if (parameters != null && getUser().getCustomer() != null) {
				customerGatewayDAO.refreshCustomerFromDatabase(getUser());
				firstAddress = getUser().getCustomer().getFirstAddress();
				if (updateCustomer != null && updateCustomer.equals("true")) {
					if (log.isDebugEnabled())
						log.debug("Updating customer account, customerID: "
								+ getUser().getCustomer().getCustomerID());
					BeanUtils.copyProperties(getUser().getCustomer(),
							parameters);
					if (!usingAddressBook) {
						// If you don't remove it, the copy will overwrite the
						// firstDelivery.customerAddressID and always add a new
						// row to the CustomerAddress table
						parameters.remove("customerAddressID");
						parameters.remove("customerBillingAddressID");

						// Also remove any fields starting with "billing" so those are not copied into the
						// CustomerAddress. With the address book, CustomerAddress now has billing fields. These
						// shouldn't be copied from the parameters.
						Iterator i = parameters.keySet().iterator();
						while (i.hasNext()) {
							String key = (String) i.next();
							if (key.startsWith("billing")) {
								i.remove();
							}
						}
						if (firstAddress == null) {
							if (log.isDebugEnabled())
								log
										.debug("Inserting customer delivery address.");
							isDeliveryAddressNew = true;
							firstAddress =  getBusinessObjectFactory()
									.createObject(CustomerAddress.class);
							BeanUtils.copyProperties(firstAddress, parameters);
							firstAddress.setCustomer(getUser().getCustomer());
							if (getUser().getCustomer().getCustomerAddressCollection() == null) {
								getUser().getCustomer().setCustomerAddressCollection(new LinkedHashSet());
							}
							getUser().getCustomer()
									.getCustomerAddressCollection().add(
											firstAddress);
						} else {
							if (log.isDebugEnabled())
								log
										.debug("Updating customer delivery address.");
							BeanUtils.copyProperties(firstAddress, parameters);
						}
					}
				}
			}
			// Update order and order delivery in the database.
			OrderGatewayDAO orderGatewayDAO =  getDaoFactory()
					.createDAO(OrderGatewayDAO.class);
			orderGatewayDAO.updateOrderAndDelivery(true, getUser().getOrder(),
					getUser().getOrder().firstDelivery());
			// Update customer account
			if (getUser().getCustomer() != null && updateCustomer != null
					&& updateCustomer.equals("true")) {
				if (isDeliveryAddressNew) {
					customerGatewayDAO.updateCustomerInsertAddress(getUser()
							.getCustomer(), firstAddress);
				} else {
					customerGatewayDAO.updateCustomerAndAddress(getUser()
							.getCustomer(), firstAddress);
				}
			}
		} catch (Exception e) {
			if (getUser().getOrder() != null
					&& getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus(corruptedStatus);
			}
			throw e;
		}
		return null;
	}

	public Map processOrderComplete(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing order on completion.");
		if (log.isInfoEnabled())
			log.info("Processing purchased gift certificates.");

		// Processed purchased gift certificates.
		String giftCertificateAmountAttributeCode = (String) getSettings()
				.getValue("giftCertificateAmountAttributeCode");
		if (giftCertificateAmountAttributeCode == null || giftCertificateAmountAttributeCode.length() == 0) {
			giftCertificateAmountAttributeCode = "GiftCertificateAmount";
		}
		if (getUser().getOrder() != null
				&& getUser().getOrder().firstDelivery() != null
				&& getUser().getOrder().firstDelivery()
						.getOrderItemCollection() != null) {
			Iterator i = getUser().getOrder().firstDelivery()
					.getOrderItemCollection().iterator();
			while (i.hasNext()) {
				OrderItem oi = (OrderItem) i.next();
				if (oi.getOrderItemAttributes() != null) {
					Iterator j = oi.getOrderItemAttributes().iterator();
					while (j.hasNext()) {
						OrderItemAttributeBean oia = (OrderItemAttributeBean) j
								.next();
						if (oia.getAttributeCode().equals(
								giftCertificateAmountAttributeCode)) {
							createGiftCertificates(oi);
						}
					}
				}
			}
		}

		// Process inventory and discounts
		if (log.isDebugEnabled())
			log.debug("Processing inventory and discounts.");
		Map results = new HashMap();
		String trackInventory = (String) getSettings().getValue(
				"trackInventory");
		if (trackInventory.equals("1")) {
			InventoryProcessor inventoryProcessor =  getBusinessObjectFactory()
					.createObject(InventoryProcessor.class);
			results = inventoryProcessor.processOrderComplete(parameters);
		}
		CartDiscountProcessor discountProcessor =  getBusinessObjectFactory()
				.createObject(CartDiscountProcessor.class);
		Map discountResults = discountProcessor
				.processOrderComplete(parameters);
		if (discountResults != null && results != null) {
			results.putAll(discountResults);
		}

		// Update totals, subtracting gift certificates
		updateOrderTotals();
		getUser().getOrder().addFormatting(getUser().getLocale(),
				(String) getSettings().getValue("currencyCode"));

		// Update the database
		OrderGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(OrderGatewayDAO.class);
		gatewayDAO.processOrderComplete(getUser().getOrder(), results);
		return results;
	}

	public Map markOrderComplete(Map parameters) throws Exception {
		Map results = new HashMap();

		// For backwards compatibility; in old versions processOrderComplete
		// also marked the order complete. If that's occurred, skip it here.
		if (getUser().getOrder().getCompleted() != null
				&& getUser().getOrder().getOrderNumber() != null) {
			return results;
		}

		// Mark the timestamp of the order
		if (log.isInfoEnabled())
			log.info("Setting the order number, timestamp, "
					+ "and the order status.");
		getUser().getOrder().setCompleted(formatDateTime(new java.util.Date()));

		// Mark the order status as received
		if (log.isDebugEnabled())
			log.debug("Marking order received.");
		getUser().getOrder().setStatus(receivedStatus);

		// Set the order number
		OrderGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(OrderGatewayDAO.class);
		OrderNumber orderNumber =  getBusinessObjectFactory()
				.createObject(OrderNumber.class);
		gatewayDAO.insertOrderNumber(orderNumber, false);
		int orderNumberFloor = Integer.parseInt((String) getSettings()
				.getValue("orderNumberFloor"));
		getUser().getOrder().setOrderNumber(
				new Integer(orderNumber.getOrderNumber() + orderNumberFloor));
		if (log.isInfoEnabled())
			log.info("Setting order number for new order: "
					+ getUser().getOrder().getOrderNumber());

		// Update the database
		gatewayDAO.markOrderComplete(getUser().getOrder(), results);

		// Call subscribed events
		NewOrderEvent noe =  getBusinessObjectFactory().createObject(NewOrderEvent.class);
		noe.setCartProcessor(this);
		getEventBus().post(noe);

		return results;

	}

	public void createGiftCertificates(OrderItem oi) throws Exception {
		String applyDiscounts = (String) getSettings().getValue(
				"applyDiscounts");
		if (applyDiscounts == null || applyDiscounts.equals("0")) {
			return;
		}

		// Create a Map of the gift cert's fields
		Double eachGiftCertTotal = utils().roundForMoney(
				oi.getTotal().doubleValue() / oi.getQuantity());
		Map gcMap = new HashMap();
		gcMap.put("name", "Gift Certificate");
		gcMap.put("itemsAppliesType", "[giftCertificate]");
		gcMap.put("rewardType", "[amount]");
		gcMap.put("reward", eachGiftCertTotal.toString());
		gcMap.put("usersAppliesType", "[allUsers]");
		gcMap.put("maxTimesUsed", "0");
		gcMap.put("isTimesPerCustomer", new Boolean(false));
		gcMap.put("isActive", new Boolean(true));
		gcMap.put("orderItem", oi);

		// Compute the expiration date based on a setting
		gcMap.put("expirationDate", computeGiftCertificateExpDate());

		// Add one gift certificate for each quantity
		for (int i = 0; i < oi.getQuantity(); i++) {

			// Create a unique code
			gcMap.put("code", RandomStringUtils.randomAlphanumeric(8)
					.toUpperCase());
			gcMap.put("couponCode", gcMap.get("code"));

			// Add the record to the database
			if (log.isInfoEnabled())
				log.info("Creating gift certificate: " + gcMap);
			DiscountProcessor processor =  getBusinessObjectFactory()
					.createObject(DiscountProcessor.class);
			Map results = processor.addDiscount(gcMap);
			if (results.get("newRecord") != null) {

				// If successful, store the discount with the order item so we
				// can retrieve it for the notification emails and thank you
				// page
				if (oi.getDiscounts() == null) {
					oi.setDiscounts(new LinkedHashSet());
				}
				oi.getDiscounts().add(results.get("newRecord"));
			}
		}
	}

	public String computeGiftCertificateExpDate() {
		// This method is to get a gift certificate exp date
		// It is only used during the order finalization process
		String expDate = "";
		String expDays = (String) getSettings().getValue(
				"giftCertificateExpireInDays");
		// Check for zero for unlimited.
		if (expDays != null && !expDays.trim().equals("")
				&& !expDays.trim().equals("0")) {
			Calendar cal = Calendar.getInstance();
			int expires = Integer.parseInt(expDays);
			cal.add(Calendar.DATE, expires);
			expDate = formatDateTime(new java.util.Date(cal.getTimeInMillis()));
		}
		return expDate;
	}

	public Map postProcessAddItems(Collection newOrderItems) throws Exception {
		return null;
	}

	public Map postProcessCartChanged() throws Exception {
		return null;
	}

	@Override
	public Map sendOrderCompleteEmails(Order order, Map parameters) throws Exception {
		Map<String, Object> results = new HashMap<String, Object>();
		EmailProcessor emailProcessor = getBusinessObjectFactory().createObject(EmailProcessor.class);
		FormUtils formUtils = (FormUtils) getBusinessObjectFactory().createUtilsObject("formUtilsImplementer");
		order.addFormatting(Locale.getDefault(), (String) getSettings().getValue("currencyCode"));
		if (formUtils.isValidEmail(order.getEmail1()) && (parameters == null
				|| parameters.get("sendCustomerEmail") == null || "true".equals(parameters.get("sendCustomerEmail")))) {
			String toEmails = order.getEmail1();
			String ccEmails = getSettings().getValue("invoiceEmailCC");
			String bccEmails = getSettings().getValue("invoiceEmailBCC");
			boolean success = emailProcessor.sendOrderEmail(order, "orderConfirmation", toEmails, ccEmails, bccEmails,
					null);
			results.put("orderConfirmationEmailResult", success);
		}
		if (parameters.get("sendNotificationEmail") == null || "true".equals(parameters.get("sendNotificationEmail"))) {
			String toEmails = getSettings().getValue("notificationEmailTO");
			if (StringUtils.isNotBlank(toEmails)) {
				String ccEmails = getSettings().getValue("notificationEmailCC");
				String bccEmails = getSettings().getValue("notificationEmailBCC");
				boolean success = emailProcessor.sendOrderEmail(order, "orderNotification", toEmails, ccEmails,
						bccEmails, null);
				results.put("orderNotificationEmailResult", success);
			}
		}
		if (parameters.get("sendGiftCertificateEmails") == null
				|| "true".equals(parameters.get("sendGiftCertificateEmails"))) {
			String ccEmails = getSettings().getValue("giftCertificateNotificationCC");
			String bccEmails = getSettings().getValue("giftCertificateNotificationBCC");
			String giftCertificateAmountAttributeCode = getSettings().getValue("giftCertificateAmountAttributeCode");
			for (OrderItem item : (Collection<OrderItem>) order.firstDelivery().getOrderItemCollection()) {
				String toEmails = order.firstDelivery().getEmail1();

				// Parse the gift certificate parameters from the order item's attributes
				Map attributeMap = utils().buildOrderItemAttributeMap((OrderItem) item);
				if (attributeMap != null && attributeMap.get(giftCertificateAmountAttributeCode) != null) {

					// Figure out the to email
					if (attributeMap != null && attributeMap.get("GiftCertificateEmail") != null) {
						OrderItemAttribute oia = (OrderItemAttribute) attributeMap.get("GiftCertificateEmail");
						if (oia.getAttributeValue() != null && oia.getAttributeValue().length() > 0) {
							toEmails = oia.getAttributeValue();
						}
					}
					if (toEmails != null && toEmails.length() > 0 && formUtils.isValidEmail(toEmails.trim())) {
						Map<String, Object> extraTemplateObjects = new HashMap<String, Object>();
						extraTemplateObjects.put("item", item);
						extraTemplateObjects.put("attributeMap", attributeMap);
						extraTemplateObjects.put("giftCertificateAmountAttribute",
								attributeMap.get(giftCertificateAmountAttributeCode));
						boolean success = emailProcessor.sendGiftCertificateEmail(order, "giftCertificate", toEmails,
								ccEmails, bccEmails, extraTemplateObjects);
						results.put("giftCertificateEmailResultForItem-" + item.getCode(), success);
					}
				}
			}
		}
		return results;
	}
}
