package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with order settings.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderSettingProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderSettingProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderSettingProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderSettingProcessor}.
 * 
 * @author Jason McWilliams
 */
public interface OrderSettingProcessor extends BusinessProcessor {

	/**
	 * Adds an order setting directly by using only parameters in the map which
	 * should also contain the orderID param. This method would usually be used
	 * from a form that only has the order setting info. This would require an
	 * refresh of a connected order object for it to show in the orderSettings
	 * property
	 * 
	 * @param parameters
	 *            Map that should include the orderSetting fields including the
	 *            orderID
	 * @throws Exception
	 */
	abstract void addOrderSetting(Map parameters) throws Exception;

	/**
	 * Loads a disconnected OrderSetting bean. This does not actually return the
	 * connected OrderSetting, but rather a copy
	 * 
	 * @param parameters
	 * @return
	 * @throws Exception
	 */
	abstract OrderSetting loadOrderSettingFromID(Map parameters)
			throws Exception;

	/**
	 * Loads a connected OrderSetting bean from the database.
	 * 
	 * @param parameters
	 * @return
	 * @throws Exception
	 */
	abstract OrderSetting loadOrderSettingObjectFromID(Map parameters)
			throws Exception;

	/**
	 * Updates an order setting in the db using the parameters given. One would
	 * have to refresh the order object to get this updated information as it is
	 * not connected
	 * 
	 * @param parameters
	 * @throws Exception
	 */
	abstract void editOrderSetting(Map parameters) throws Exception;

	/**
	 * Deletes an order setting from the database. One would have to refresh the
	 * order object to get this updated information as it is not connected.
	 * 
	 * @param parameters
	 * @throws Exception
	 */
	abstract void deleteOrderSetting(Map parameters) throws Exception;

	/**
	 * Find the order setting by code for a given order
	 * 
	 * @param order
	 *            The Order under which the OrderSetting is found
	 * @param code
	 *            The code of the OrderSetting
	 * @throws Exception
	 */
	abstract OrderSetting findOrderSettingByCode(Order order, String code)
			throws Exception;

	/**
	 * Adds or updates an orderSetting for a given order. The order should be a
	 * connected object while the OrderSetting should be treated as a simple
	 * transfer object (TO) without setOrderID or setOrder actually set. If the
	 * order contains the orderSetting.code, the data will be updated otherwise
	 * the a new setting will be added.
	 * 
	 * @param order
	 * @param orderSetting
	 * @param commitOrder
	 * @throws Exception
	 */
	abstract void addOrUpdateOrderSetting(Order order,
			OrderSetting orderSetting, boolean commitOrder) throws Exception;
}
