package com.softslate.commerce.customer.customer;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.octo.captcha.module.config.CaptchaModuleConfigHelper;
import com.octo.captcha.module.struts.CaptchaServicePlugin;
import com.octo.captcha.service.CaptchaService;
import com.softslate.commerce.businessobjects.core.RecaptchaProcessor;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions to register a new customer account. Used for requests to "/Register.do".
 * <p>
 * The <code>validate</code> method of this class checks for the existance of the <code>userName</code> and
 * <code>decryptedPassword</code> properties, and checks that the <code>decryptedPassword</code> property matches the
 * <code>confirmPassword</code> property.
 * <p>
 * It also checks the length of the password against the "minimumPasswordLength" database setting. If the
 * "keepUserNameAndEmailInSync" setting is on, it will verify that the user name is a valid email address and copy the
 * user name to the <code>email1</code> property.
 * <p>
 * If the "useImageVerificationWithRegistrations" setting is set, it also checked the entered text against the Jcaptcha
 * image that was presented.
 * 
 * @author David Tobey
 */
public class RegisterForm extends BaseForm {
	private static final long serialVersionUID = 3258131362296115257L;

	static Log log = LogFactory.getLog(RegisterForm.class);

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String decryptedPassword = null;

	public String getDecryptedPassword() {
		return (this.decryptedPassword);
	}

	public void setDecryptedPassword(String decryptedPassword) {
		this.decryptedPassword = decryptedPassword;
	}

	private String confirmPassword = null;

	public String getConfirmPassword() {
		return (this.confirmPassword);
	}

	public void setConfirmPassword(String confirmPassword) {
		this.confirmPassword = confirmPassword;
	}

	private String emailerOptInApproval = "Yes";

	public String getEmailerOptInApproval() {
		return emailerOptInApproval;
	}

	public void setEmailerOptInApproval(String emailerOptInApproval) {
		this.emailerOptInApproval = emailerOptInApproval;
	}

	private String textFromImage = null;

	public String getTextFromImage() {
		return textFromImage;
	}

	public void setTextFromImage(String textFromImage) {
		this.textFromImage = textFromImage;
	}

	private String originalRequest = null;

	public String getOriginalRequest() {
		return originalRequest;
	}

	public void setOriginalRequest(String originalRequest) {
		this.originalRequest = originalRequest;
	}

	private String csrfToken = null;

	public String getCsrfToken() {
		return csrfToken;
	}

	public void setCsrfToken(String csrfToken) {
		this.csrfToken = csrfToken;
	}

	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
		}
		// Validate the user name
		if (getUserName() != null) {
			setUserName(getUserName().trim());
		}
		if (getUserName() == null || getUserName().length() == 0) {
			String message = getMessage("customer", "customer.userName", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
		}

		// If the system is set to require email activation, a password is not required. Instead, we set the password
		// to an unguessable GUID, and later we'll send an email to the customer to reset the password.
		boolean requireEmailActivation = ("1").equals(getSettings().getValue("requireEmailActivation"));
		if (requireEmailActivation) {
			setDecryptedPassword(java.util.UUID.randomUUID().toString());
		} else {
			// Set the minimum password length
			int minimumPasswordLength = 0;
			try {
				String minimumPasswordLengthString = (String) getSettings().getValue("minimumPasswordLength");
				minimumPasswordLength = Integer.parseInt(minimumPasswordLengthString);
			} catch (Exception e) {
				log.error(e.toString(), e);
			}
			// Validate the password
			if (getDecryptedPassword() == null || getDecryptedPassword().length() < minimumPasswordLength) {
				String message = getMessage("customer", "customer.password", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.minlength", message, Integer.toString(minimumPasswordLength)));
			}
			// Check if the password requires a letter and a number
			boolean passwordRequiresLetterAndNumberCustomer = ("1")
					.equals(getSettings().getValue("passwordRequiresLetterAndNumberCustomer"));
			if (passwordRequiresLetterAndNumberCustomer && (!getDecryptedPassword().matches(".*[a-zA-Z].*")
					|| !getDecryptedPassword().matches(".*[0-9].*"))) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.passwordDoesNotHaveLettersAndNumbers"));
			}
			// Check that the password and confirm password match.
			if (getDecryptedPassword() != null && getConfirmPassword() != null
					&& !getDecryptedPassword().equals(getConfirmPassword())) {
				String message1 = getMessage("customer", "customer.password", null);
				String message2 = getMessage("customer", "customer.confirmPassword", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.mustMatch", message1, message2));
			}
		}
		// If the user name and email are to be kept in sync, copy the user name
		// over and verify it is a valid email address.
		try {
			String keepUserNameAndEmailInSync = (String) getSettings().getValue("keepUserNameAndEmailInSync");
			if (keepUserNameAndEmailInSync.equals("1")) {
				setEmail1(getUserName());
				if (!formUtils().isValidEmail(getEmail1())) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.invalidEmail", getEmail1()));
				}
				// Otherwise, copy over the email1 property, so this form can be
				// used for
				// with logged-in customers to update their info.
			} else if (getUser().getCustomer() != null && getUser().getCustomer().getEmail1() != null) {
				setEmail1(getUser().getCustomer().getEmail1());
			}
		} catch (Exception e) {
			log.error(e.toString(), e);
		}

		// Validate CAPTCHA
		try {
			String useLegacyCaptchaMethod = (String) getSettings().getValue("useLegacyCaptchaMethod");
			String useImageVerificationWithRegistrations = (String) getSettings().getValue(
					"useImageVerificationWithRegistrations");
			if (getErrors().isEmpty() && useImageVerificationWithRegistrations != null
					&& useImageVerificationWithRegistrations.equals("1") && useLegacyCaptchaMethod != null
					&& useLegacyCaptchaMethod.equals("1")) {

				// Legacy JCaptcha method
				if (getTextFromImage() != null) {
					setTextFromImage(getTextFromImage().toLowerCase());
				}
				CaptchaService service = CaptchaServicePlugin.getInstance().getService();
				String captchaID = CaptchaModuleConfigHelper.getId(request);
				if (captchaID == null || getTextFromImage() == null || getTextFromImage().length() == 0
						|| !service.validateResponseForID(captchaID, getTextFromImage()).booleanValue()) {
					setTextFromImage(null);
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidCaptcha"));
				}
			} else if (getErrors().isEmpty() && useImageVerificationWithRegistrations != null
					&& useImageVerificationWithRegistrations.equals("1")) {

				// Google Recaptcha method
				RecaptchaProcessor rp = getBusinessObjectFactory().createObject(RecaptchaProcessor.class);
				Map<String, String> params = new HashMap();
				params.put("response", request.getParameter("g-recaptcha-response"));
				Map response = rp.verifyRecaptcha(params);
				if (response == null || response.get("success") == null
						|| response.get("success").equals(Boolean.FALSE)) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
				}
			}

		} catch (Exception e) {
			log.error(e.toString(), e);
			setTextFromImage(null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
		}

		// Set the emailOptInApproval property to no if it was not actually
		// submitted.
		if (request.getParameter("emailerOptInApproval") == null) {
			setEmailerOptInApproval("No");
		}
		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
