package com.softslate.commerce.daos.product;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Filter;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscCategory</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class CategoryDAOHibernate extends BaseDAO implements CategoryDAO {

	static Log log = LogFactory.getLog(CategoryDAOHibernate.class);

	private Category category = null;

	public Category getCategory() {
		return (this.category);
	}

	public void setCategory(Category category) {
		this.category = category;
	}

	public void loadCategoryFromID() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading category: " + getCategory().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Category category = (Category) session.get(Category.class,
					new Integer(getCategory().getCategoryID()));
			if (category != null) {
				PropertyUtils.copyProperties(getCategory(), category);
			}
			setCategory(category);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadCategory() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectFromCode");
			q.setString("code", getCategory().getCode());
			q.setCacheable(true);
			Category loadedCategory = (Category) q.uniqueResult();
			setCategory(loadedCategory);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadCategoryFromCode() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectCategoryFromCode");
			q.setString("code", getCategory().getCode());
			Category loadedCategory = (Category) q.uniqueResult();
			setCategory(loadedCategory);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	/**
	 * Construct the parentString and parentCollection properties by looping
	 * recursively up the category tree.
	 */
	public void loadParents() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading category parents: " + getCategory().getCode());
		StringBuffer parentStringBuffer = new StringBuffer();
		ArrayList parentArray = new ArrayList();
		if (getCategory().getParentCategory() != null
				&& getCategory().getParentCategory().intValue() == getCategory()
						.getCategoryID()) {
			if (log.isWarnEnabled())
				log.warn("Category " + getCategory().getCode()
						+ " is in the database as a parent of itself!");
		} else {

			// Limit the number of levels the parent categories recurse to avoid
			// an infinite loop
			int maxRecordsProcessed = 100;
			try {
				maxRecordsProcessed = Integer.parseInt((String) getSettings()
						.getValue("maxRecordsProcessed"));
			} catch (Exception e) {
				log.warn("maxRecordsProcessed error, defaulting to 100: ", e);
			}
			int count = 0;

			// Load parents recursively
			Category parent = getCategory().getParent();
			while (parent != null) {
				if (count >= maxRecordsProcessed) {
					log.warn("Max number of category levels reached ("
							+ maxRecordsProcessed + ") with: "
							+ parent.getCode()
							+ ". Check for circular references.");
					break;
				}
				int currentParentID = parent.getCategoryID();
				parentArray.add(0, parent);
				parentStringBuffer.append(","
						+ Integer.toString(currentParentID) + ",");
				if (parent.getParentCategory() != null
						&& parent.getParentCategory().intValue() == parent
								.getCategoryID()) {
					if (log.isWarnEnabled())
						log.warn("Category " + parent.getCode()
								+ " is in the database as a parent of itself!");
					parent = null;
				} else {
					count++;
					parent = parent.getParent();
				}
			}
		}
		getCategory().setParentCollection(parentArray);
		getCategory().setParentString(parentStringBuffer.toString());
		if (log.isDebugEnabled())
			log.debug("parentString: " + getCategory().getParentString());
		return;
	}

	public void loadSubcategories() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading subcategories: " + getCategory().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.selectSubcategories");
			q.setInteger("categoryID", getCategory().getCategoryID());
			q.setCacheable(true);
			getCategory().setSubcategoryCollection(new LinkedHashSet(q.list()));
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertCategory() throws DataAccessException {
		insertCategory(false);
	}

	public void insertCategory(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(getCategory());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateCategory() throws DataAccessException {
		updateCategory(false);
	}

	public void updateCategory(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getCategory());
			if (commit) {
				getDaoFactory().commitTransaction();
				if (getCategory() != null) {
					session.evict(getCategory());
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateCategoryOrderByID(int categoryID, int categoryOrder)
			throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "category.updateCategoryOrderByID");
			q.setInteger("categoryID", categoryID);
			q.setInteger("categoryOrder", categoryOrder);
			q.executeUpdate();
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return;
	}

	public void deleteCategory() throws DataAccessException {
		deleteCategory(false);
	}

	public void deleteCategory(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setCategory((Category) session.load(Category.class, new Integer(
					getCategory().getCategoryID())));
			session.delete(getCategory());
			CategoryGatewayDAO dao =  getDaoFactory()
					.createDAO(CategoryGatewayDAO.class);
			String[] deletedIDs = new String[] { Integer.toString(getCategory()
					.getCategoryID()) };
			dao.updateNullifyParentIDs(deletedIDs);
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (Exception he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Collection loadSettings() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		Collection results = null;
		try {
			Query q = getNamedQuery(session, "categorySetting.selectAll");
			q.setInteger("categoryID", getCategory().getCategoryID());
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return results;
	}

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof Category) {
			this.setCategory((Category) object);
			if (0 != category.getCategoryID()) {
				this.loadCategoryFromID();
			} else if (null != category.getCode()) {
				this.loadCategoryFromCode();
			}
			return this.getCategory();
		}
		return null;
	}

	public boolean deleteObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Category) {
			this.setCategory((Category) object);
			this.deleteCategory(commit);
			return true;
		}
		return false;
	}

	public Object insertObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Category) {
			this.setCategory((Category) object);
			this.insertCategory(commit);
			return this.getCategory();
		}
		return null;
	}

	public Object updateObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Category) {
			this.setCategory((Category) object);
			this.updateCategory(commit);
			return this.getCategory();
		}
		return null;

	}

	public Collection loadAll() throws Exception {
		return null;
	}

	public Collection loadAll(Filter filter) throws Exception {
		return null;
	}

}
